// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <string>
#include <map>
#include <vector>

#include <aoc/common/Graph.hpp>

class LabelGraph
{
public:
    int addVertex(const std::string& label);
    void addEdge(const int vertex1, const int vertex2);
    bool areAdjacent(const int vertex1, const int vertex2) const;
    const std::string& getVertexLabel(const int vertex) const;
    const std::map<std::string, int>& getLabelVertices() const;
    Graph::NeighborIterator begin(const int vertex) const;
    Graph::NeighborIterator end() const;
private:
    Graph graph_{};
    std::vector<std::string> vertexLabels_{};
    std::map<std::string, int> labelVertices_{};
};
