// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/common/Graph.hpp>

int Graph::addVertex()
{
    firstVertexIncidences_.push_back(-1);
    return static_cast<int>(firstVertexIncidences_.size()) - 1;
}

void Graph::addEdge(const int vertex1, const int vertex2)
{
    incidenceVertices_.push_back(vertex2);
    nextIncidences_.push_back(firstVertexIncidences_[vertex1]);
    firstVertexIncidences_[vertex1] = static_cast<int>(incidenceVertices_.size()) - 1;

    incidenceVertices_.push_back(vertex1);
    nextIncidences_.push_back(firstVertexIncidences_[vertex2]);
    firstVertexIncidences_[vertex2] = static_cast<int>(incidenceVertices_.size()) - 1;
}

size_t Graph::getNVertices() const
{
    return firstVertexIncidences_.size();
}

size_t Graph::getNEdges() const
{
    return nextIncidences_.size() >> 1;
}

bool Graph::areAdjacent(const int vertex1, const int vertex2) const
{
    auto it = begin(vertex1);
    while (it != end())
    {
        if (it->vertex == vertex2)
        {
            return true;
        }
        it++;
    }
    return false;
}

Graph::NeighborIterator Graph::begin(const int vertex) const
{
    return { *this, vertex };
}

Graph::NeighborIterator Graph::end() const
{
    return { *this, -1 };
}
