// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/common/LabelGraph.hpp>

int LabelGraph::addVertex(const std::string& label)
{
    vertexLabels_.push_back(label);
    labelVertices_.insert({ label, static_cast<int>(vertexLabels_.size()) - 1 });
    return graph_.addVertex();
}

void LabelGraph::addEdge(const int vertex1, const int vertex2)
{
    graph_.addEdge(vertex1, vertex2);
}

bool LabelGraph::areAdjacent(const int vertex1, const int vertex2) const
{
    return graph_.areAdjacent(vertex1, vertex2);
}

const std::string& LabelGraph::getVertexLabel(const int vertex) const
{
    return vertexLabels_[vertex];
}

const std::map<std::string, int>& LabelGraph::getLabelVertices() const
{
    return labelVertices_;
}

Graph::NeighborIterator LabelGraph::begin(const int vertex) const
{
    return graph_.begin(vertex);
}

Graph::NeighborIterator LabelGraph::end() const
{
    return graph_.end();
}
